import { api } from "dicomweb-client";
import dcmjs from "dcmjs";
import { utilities } from "@cornerstonejs/core";
import cornerstoneDICOMImageLoader from "@cornerstonejs/dicom-image-loader";
import getPixelSpacingInformation from "./getPixelSpacingInformation";
import { convertMultiframeImageIds } from "./convertMultiframeImageIds";
import removeInvalidTags from "./removeInvalidTags";

const { DicomMetaDictionary } = dcmjs.data;
const { calibratedPixelSpacingMetadataProvider } = utilities;

interface CreateImageIdsAndCacheMetaDataOptions {
  StudyInstanceUID: string;
  SeriesInstanceUID: string;
  SOPInstanceUID?: string | null;
  wadoRsRoot: string;
  client?: api.DICOMwebClient | null;
}

const SOP_INSTANCE_UID = "00080018";
const SERIES_INSTANCE_UID = "0020000E";
// 数据扫描设备 CT、CTA
const MODALITY = "00080060";

/**
 * 从 DICOMweb 服务创建图像 ID 并缓存相关元数据。
 *
 * @param options - 包含图像和 DICOMweb 服务信息的对象
 * @returns 图像 ID 数组
 */
export default async function createImageIdsAndCacheMetaData(
  options: CreateImageIdsAndCacheMetaDataOptions
): Promise<string[]> {
  const studySearchOptions = {
    studyInstanceUID: options.StudyInstanceUID,
    seriesInstanceUID: options.SeriesInstanceUID,
  };

  const client =
    options.client ||
    new api.DICOMwebClient({ url: options.wadoRsRoot, singlepart: true });

  try {
    // TODO: 取第一张的metadata看下扫描设备，加上限制，如果不满足条件，就别缓存了
    const instances = await client.retrieveSeriesMetadata(studySearchOptions);

    const modality = (instances[0] as unknown as any)[MODALITY].Value[0];
    console.log("modality", modality);
    let imageIds = instances.map((instanceMetaData: any) => {
      const SeriesInstanceUID = instanceMetaData[SERIES_INSTANCE_UID].Value[0];
      const SOPInstanceUIDToUse =
        options.SOPInstanceUID || instanceMetaData[SOP_INSTANCE_UID].Value[0];

      const prefix = "wadors:";
      const imageId = `${prefix}${options.wadoRsRoot}/studies/${options.StudyInstanceUID}/series/${SeriesInstanceUID}/instances/${SOPInstanceUIDToUse}/frames/1`;

      cornerstoneDICOMImageLoader.wadors.metaDataManager.add(
        imageId,
        instanceMetaData
      );
      return imageId;
    });

    // 如果图像 ID 代表多帧信息，则为每个帧创建一个新的图像 ID 列表
    imageIds = convertMultiframeImageIds(imageIds);

    imageIds.forEach((imageId) => {
      let instanceMetaData =
        cornerstoneDICOMImageLoader.wadors.metaDataManager.get(imageId);
      instanceMetaData = removeInvalidTags(instanceMetaData);

      if (instanceMetaData) {
        // 添加校准的像素间距
        const metadata =
          DicomMetaDictionary.naturalizeDataset(instanceMetaData);
        const pixelSpacing = getPixelSpacingInformation(metadata) as Number[];
        if (pixelSpacing) {
          const payload = {
            rowPixelSpacing: pixelSpacing[0],
            columnPixelSpacing: pixelSpacing[1],
          };
          // FIXME: 这个地方cornerstone ts类型约束洗的有问题
          // @ts-ignore
          calibratedPixelSpacingMetadataProvider.add(imageId, payload);
        }
      }
    });
    return imageIds;
  } catch (error) {
    throw new Error("pacs中数据不存在");
  }
}
